import typer
from typing import Optional
from pathlib import Path
from . import edgar, models, config, utils, setup
from pyseek import __app_name__, __version__, SUCCESS

app = typer.Typer()


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()


def validate_ticker_or_cik(company: str) -> models.CIK:
    """Validate the ticker or CIK number

    Args:
        company (str): either the company ticker or the company cik number in str format

    Raises:
        typer.BadParameter: _description_
        typer.BadParameter: _description_

    Returns:
        models.CIK: company information with keys "ticker", "name", "cik_str"
    """
    try:
        company = int(company)
        validation = utils.company_from_cik(company)
        if not validation:
            raise typer.BadParameter(f"No results found for CIK {company}")
        return models.CIK(**validation[0])
    except ValueError:
        result = utils.company_from_ticker(company)
        if result:
            return models.CIK(**result[0])
        else:
            raise typer.BadParameter(f"No results found for ticker {company}")


@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit.",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return


@app.command()
def init(
    user_agent: str = typer.Option(
        ...,
        "--user-agent",
        "-u",
        help="User agent for the package. Use the format <name> (<email>)",
        prompt="Please enter a user-agent for browsing SEC EDGAR website",
    ),
    download: bool = typer.Option(
        True,
        " /--download",
        " /-d",
        help="Download company tickers information to configuration directory",
        prompt="Download company tickers information to configuration directory",
    ),
):
    """Initialize the user settings"""
    result = config.init_config(user_agent)
    if result == SUCCESS:
        typer.echo("Configuration file created successfully")
        if download:
            typer.echo(f"Downloading company tickers information")
            tickers = edgar.get_cik_numbers()
            config.create_file(filename="company_tickers.json")
            utils.write_file(
                tickers,
                "company_tickers.json",
                directory=setup.CONFIGURATION_DIRECTORY,
            )


@app.command("settings")
def api_settings() -> dict:
    """Get the current settings"""
    apisettings = config.get_api_settings()
    print(list(apisettings.items()))


@app.command()
def get_cik(
    ticker: str,
    use_stored: bool = typer.Option(
        True,
        " /--stored",
        " /-s",
        help="Use the stored company ticker files or retrieve a live list from sec website",
    ),
) -> list[models.CIK]:
    """Get the CIK number for a given ticker"""
    if use_stored:
        print(utils.company_from_ticker(ticker))
    else:
        print(edgar.get_cik_number(ticker))


@app.command()
def company_facts(
    company: str = typer.Argument(..., help="CIK number or ticker of the company"),
) -> dict:
    """Get all the company facts for a given company"""
    company = validate_ticker_or_cik(company)
    result = edgar.get_all_company_facts(company.cik_str)
    utils.write_file(result, company.ticker + "_facts.json")


@app.command()
def company_concept_categories(
    company: str = typer.Argument(..., help="CIK number or ticker of the company"),
) -> list:
    """Get all the company concepts categories for a given CIK"""
    company = validate_ticker_or_cik(company)
    print(edgar.get_company_concepts_categories(company.cik_str))


@app.command()
def company_facts_by_concept(
    company: str = typer.Argument(..., help="CIK number or ticker of the company"),
    concept: str = typer.Argument(..., help="The concept category to retrieve"),
) -> dict:
    """Returns all facts related to a company concept category

    Args:
        company (str, optional): _description_. Defaults to typer.Argument(..., help='CIK number or ticker of the company').
        concept (str, optional): _description_. Defaults to typer.Argument(..., help='The concept category to retrieve').

    Raises:
        typer.BadParameter: _description_

    Returns:
        dict: _description_
    """
    company = validate_ticker_or_cik(company)
    print(edgar.get_company_facts_by_concept(company.cik_str, concept))


@app.command()
def company_submissions(
    company: int = typer.Argument(..., help="CIK number or ticker of the company"),
) -> dict:
    """Get all the company submissions for a given CIK"""
    company = validate_ticker_or_cik(company)
    results = edgar.get_all_company_submissions(company.cik_str)
    config.write_file(results, company.ticker + "_submissions.json")


@app.command()
def download_submission(
    cik_number: int = typer.Argument(..., help="CIK number of the company"),
    accession_number: str = typer.Argument(
        ..., help="Accession number of the submission"
    ),
    primaryDocument: str = typer.Argument(
        ..., help="Primary document of the submission"
    ),
    filename: str = typer.Option(
        None,
        "--filename",
        "-f",
        help="The name of the file to create holding the company submission information",
    ),
) -> dict:
    """Download a company submission for a given CIK"""
    if not isinstance(cik_number, int):
        raise typer.BadParameter(
            "CIK number must be an integer, please use the get_cik command to get the CIK number"
        )

    if not filename:
        filename = f"{cik_number}_{accession_number}_{primaryDocument}.txt"

    result = edgar.download_company_submission(
        cik_number, accession_number, primaryDocument
    )
    utils.write_file(result, filename)


if __name__ == "__main__":
    app()



import json
from typing import List, TypeVar, Union
import requests
from pyseek.models import CIK
from pyseek.utils import make_request, download_document

central_index_key = TypeVar("central_index_key", str, int, CIK)


def get_cik_numbers() -> dict:
    """Download all the company ticker information from the SEC"""
    return make_request("https://www.sec.gov/files/company_tickers.json")


def get_cik_number(company_ticker: str) -> List[CIK]:
    """Central Index Key The Central Index Key (CIK) is used on the SEC's computer systems to identify corporations and individual people who have filed disclosure with the SEC.

    Args:
        company_name (str): Name of the company to search for

    Returns:
        List[dict]: List of CIK(title, ticker, cik_str)
    """
    data = make_request("https://www.sec.gov/files/company_tickers.json")
    data = [data[key] for key in data.keys()]
    results = [CIK(**res) for res in data if res["ticker"] == company_ticker]
    if len(results) == 1:
        return results[0]
    elif len(results) == 0:
        raise ValueError("No results found")
    else:
        return results


def get_all_company_submissions(cik: central_index_key) -> dict:
    """Return the entity's current filing history

    This JSON data structure contains metadata such as current name, former name, and stock exchanges and ticker symbols of publicly-traded companies. The object’s property path contains at least one year’s of filing or to 1,000 (whichever is more) of the most recent filings in a compact columnar data array. If the entity has additional filings, files will contain an array of additional JSON files and the date range for the filings each one contains.

    Args:
        cik (central_index_key): CIK, str, int

    Returns:
        dict: Filing history with metadata
    """

    return make_request(f"https://data.sec.gov/submissions/CIK{cik}.json")


def get_all_company_facts(cik: central_index_key) -> dict:
    """Returns all the company concepts data for an entity in a single call

    Args:
        cik (central_index_key): CIK, str, int

    Returns:
        dict: metadata, along with time series of different company concepts
    """
    return make_request(f"https://data.sec.gov/api/xbrl/companyfacts/CIK{cik}.json")


def get_company_concepts_categories(cik: central_index_key) -> List[str]:
    """Get the categories of concepts that are available for a given company

    Args:
        cik (central_index_key): CIK, str, int

    Returns:
        List[str]: list of the categories that are searchable
    """
    cats = get_all_company_facts(cik)
    return cats.get("facts").keys()


def get_company_facts_by_concept(cik: central_index_key, category: str) -> List[str]:
    """The SEC edgar api supports getting just the data relevant to the gaap figures

    Args:
        cik (central_index_key): central index key to look up

    Returns:
        List[str]: list of the us-gaap concepts that are searchable
    """
    facts = get_all_company_facts(cik)
    return facts.get("facts").get(category)


def get_company_fact(fact: str, cik: central_index_key) -> dict:
    """Hte xbrl/companyconcept API returns a given concept for a given entity. This API supports for annual, quarterly and instantaneous data

    Args:
        fact (str): _description_
        cik (central_index_key): _description_

    Returns:
        dict: _description_
    """ """"""
    """The xbrl/frames API aggregates one fact for each reporting entity that is last filed that most closely fits the calendrical period requested.
    This API supports for annual, quarterly and instantaneous data"""
    return make_request(f"https:/data.sec.gov/api/xbrl/companyconcept/CIK/{cik}")


def get_frames(fact: str, period: str, unit: str) -> dict:
    return make_request(
        f"https:/data.sec.gov/api/xbrl/frames/{fact}/{unit}/{period}.json"
    )


def download_company_submission(
    cik: central_index_key, accession_number: str, primaryDocument: str
) -> str:
    """Download a company submission

    Args:
        cik (central_index_key): CIK, str, int
        accession_number (str): Accession number of the submission
        primaryDocument (str): Name of the primary document

    Returns:
        str: The submission as a string
    """
    return download_document(cik, accession_number, primaryDocument)


if __name__ == "__main__":
    with open("temporary/tesla_forms.json", "r") as fp:
        data = json.load(fp)

    cik = data["cik"]
    filings = data["filings"]["recent"]
    accession_number = filings["accessionNumber"][6]
    primaryDocument = filings["primaryDocument"][6]

    r = download_company_submission(cik, accession_number, primaryDocument)
    print(r.status_code)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyseek']

package_data = \
{'': ['*']}

install_requires = \
['black>=23.1.0,<24.0.0', 'requests>=2.28.2,<3.0.0', 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['pyseek = pyseek.__main__:app']}

setup_kwargs = {
    'name': 'pyseek',
    'version': '0.2.1',
    'description': 'A command line tool for accessing the SEC EDGAR database.',
    'long_description': '',
    'author': 'Zach Lopez',
    'author_email': 'zachlopez9@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Top level package for pyseek."""

__app_name__ = "pyseek"
__version__ = "0.2.1"


(
    SUCCESS,
    DIR_ERROR,
    FILE_ERROR,
    CONFIG_ERROR,
) = range(4)

ERRORS = {
    DIR_ERROR: "Configurations directory could not be created.",
    FILE_ERROR: "Configurations file could not be created.",
    CONFIG_ERROR: "Configurations file could not be written.",
}



from dataclasses import dataclass, field
from datetime import date
from typing import Union, List


@dataclass
class CIK:
    title: str
    ticker: str
    cik_str: str

    def __post_init__(self):
        self.cik_str = str(self.cik_str)
        while len(self.cik_str) < 10:
            self.cik_str = "0" + self.cik_str


@dataclass
class Measure:
    frame: str
    filed: date
    end: date
    form: str
    fp: str
    fy: str
    val: Union[float, int]
    accn: str


@dataclass
class Concept:
    label: str
    description: str
    units: str
    measures: List[Measure] = field(default_factory=list, repr=False)



import configparser
from typing import List
from pyseek import SUCCESS, DIR_ERROR, FILE_ERROR, CONFIG_ERROR
from pathlib import Path
from pyseek import setup


def create_file(
    configuration_directory: str = setup.CONFIGURATION_DIRECTORY,
    filename: str = "config.ini",
) -> int:
    """Take a configuration directory and create a file in it.

    Args:
        configuration_directory (str): the configuration directory
        filename (str, optional): the name of the file to create. Defaults to "config.ini".

    Returns:
        int: the result code of the operation
    """
    try:
        configuration_directory = Path(configuration_directory)
        configuration_directory.mkdir(exist_ok=True)
    except OSError:
        return DIR_ERROR
    try:
        configuration_file = configuration_directory / filename
        configuration_file.touch(exist_ok=True)
    except OSError:
        return FILE_ERROR
    return SUCCESS


def init_config(user_agent: str):
    """Called from the pyseek init command
    creates a configuration directory specified at config_file_dir
    and a configuration file in that directory called config.ini.
    The configuration file is a .ini file with a section called API
    """
    f = "config.ini"
    result = create_file(filename=f)
    if result == SUCCESS:
        config = configparser.ConfigParser()
        config["API"] = {"User-Agent": user_agent}
        config["TickerUpdateFrequency"] = {"Frequency": "never"}
        try:
            with open(Path(setup.CONFIGURATION_DIRECTORY) / f, "w") as configfile:
                config.write(configfile)
        except OSError:
            return CONFIG_ERROR
        return SUCCESS
    else:
        return result


def get_api_settings() -> configparser.SectionProxy:
    """Get the API settings from the config file"""
    settings_file = Path(setup.CONFIGURATION_DIRECTORY) / "config.ini"
    config = configparser.ConfigParser()
    config.read(settings_file)
    return config["API"]


if __name__ == "__main__":
    setting = get_api_settings()
    print(setting)



import json
from typing import TypeVar

import requests
from pathlib import Path
from pyseek import models
from pyseek import setup
from pyseek import config


centralIndexKey = TypeVar("centralIndexKey", str, int, models.CIK)


def set_headers() -> dict:
    """Set the headers for the requests call"""
    settings = config.get_api_settings()
    return {"User-Agent": settings["User-Agent"]}


def make_request(url: str, requestTimeout: int = 5) -> dict:
    """Handles all the requests calls for the package

    Args:
        url (str): url to request

    Returns:
        dict: the json returned
    """
    try:
        r = requests.get(url, headers=set_headers(), timeout=requestTimeout)
        r.raise_for_status()
        return r.json()
    except requests.ConnectionError:
        print("there was a connection error")
    except requests.JSONDecodeError:
        print(f"There was a JSON decode error for url: {url}")
        print(f"Check the url for errors. If the url is correct, try again later.")


def company_from_ticker(ticker: str) -> int:
    """Use the company_tickers.json file downloaded in `pyseek init` to get the CIK number for the company

    Args:
        ticker (str): ticker to look up

    Returns:
        int: company information for a given ticker
    """
    ticker = ticker.upper()
    with open(Path(setup.CONFIGURATION_DIRECTORY) / "company_tickers.json", "r") as fp:
        data = json.load(fp)
    return [company for company in data.values() if company["ticker"] == ticker]


def company_from_cik(cik: int) -> str:
    """Use the company_tickers.json file downloaded in `pyseek init` to get the ticker for the company

    Args:
        cik (int): cik number to look up

    Returns:
        str: company information for given cik number
    """
    with open(Path(setup.CONFIGURATION_DIRECTORY) / "company_tickers.json", "r") as fp:
        data = json.load(fp)
    return [company for company in data.values() if company["cik_str"] == cik]


def download_document(
    cik: centralIndexKey, accession_number: str, primaryDocument: str
):
    """Download a company submission

    Args:
        cik (central_index_key): CIK, str, int
        accession_number (str): Accession number of the submission
        primaryDocument (str): Name of the primary document

    Returns:
        str: The submission as a string
    """
    try:
        response = requests.get(
            f"https://www.sec.gov/Archives/edgar/data/{cik}/{accession_number.replace('-', '')}/{primaryDocument}",
            headers=set_headers(),
        )
        response.raise_for_status()
    except requests.HTTPError as http_err:
        print(f"HTTP error occurred: {http_err}")
    except Exception as err:
        print(f"Other error occurred: {err}")
    else:
        print("Success!")
    return response.text


def write_file(
    obj: dict,
    filename: str,
):
    with open(filename, "w") as fp:
        json.dump(obj, fp, indent=4)
